
aglm <- function(formula, family = gaussian, data, 
                 rcol=NULL, ccol=NULL, tcol=NULL,
                 nb, bor=NULL, onlynew=FALSE, 
                 weights, start = NULL, brlr = TRUE,
                 nboot = 999,  ...) {
  nllbrlr <- function(beta) {
    pb <- drop(1/(1+dt*exp(-X%*%beta)))
    obj <- -sum(dbinom(y, 1, pb, log=TRUE))
    if (brlr)
      obj <- obj -0.5*determinant(crossprod(X*(pb*(1-pb)),X))$modulus
    return(obj)
  }
  stLag <- function(spform, tLag) {
    if (sregular) {
      id <- which(names(data)==as.character(spform)[2])
      objtolag <- tapply(data[, id],
                         lapply(data[, c(rcol,ccol,tcol)],
                                as.character), as.numeric)
      return(snrl3(spform, objtolag, tLag))
    }
    else {
      sterms <- match(strsplit(as.character(spform)[3], " ")[[1]], 
                      c("S", "first", "second")) 
      id <- which(names(data)==as.character(spform)[2])
      objtolag <- tapply(data[, id], data[,tcol], as.numeric)
      res <- list()
      dd <- c(length(objtolag), length(objtolag[[1]]))
      if (any(sterms==1, na.rm=TRUE))
        res$S <- c(unlist(objtolag[1:(dd[1]+lag)]),
                   rep(NA, dd[1]*abs(lag)))
      if (any(sterms==2, na.rm=TRUE))
        res$first <- c(unlist(lapply(objtolag[1:(dd[1]+lag)], function(x) 
                                     sapply(nb, function(j) nfun(x[[j]])))), 
                       rep(NA, dd[1]*abs(lag)))
      if (any(sterms==2, na.rm=TRUE))
        warning("'second' lag irregular spatial lattice in construction!")
      names(res) <- paste(names(data)[id], names(res), 
                          tLag, sep=".")
      return(as.matrix(as.data.frame(res)))
    }
  }
  if (is.null(bor))
    bor <- 1
  sregular <- ifelse(!is.null(rcol) & !is.null(ccol), TRUE, FALSE)
  dtemporal <- ifelse(is.null(tcol), FALSE, TRUE)
  mf <- match.call(expand.dots = FALSE)
  aglm.env <- environment(mf)
  m <- match(c("formula", "data", "subset", "weights", "na.action", 
                "offset"), names(mf), 0)
  mff <- mf[c(1, m)]
  mff$drop.unused.levels <- TRUE
  mff$na.action <- "na.pass"
  mff[[1]] <- as.name("model.frame")
  mff <- eval(mff, aglm.env)
  X <- model.matrix(attr(mff, "terms"), mff)
  y <- y.ori <- model.response(mff)
##  if (family()$family=="binomial") {
  ##  if (!is.matrix(y)) 
    ##  y <- y.ori <- cbind(y, 1) 
##  }
  if (sregular) {
    ddd <- sapply(lapply(data[, c(rcol, ccol, tcol)],
                         function(x) unique(as.character(x))), length)
    if (dtemporal) {
      dt <- dt0 <- tapply(data[, tcol], data[, c(rcol,ccol,tcol)], as.numeric)
      for (i in 2:ddd[3])
        dt[,,i] <- dt0[,,i]-dt0[,,i-1]
      dt <- dt/mean(dt[,,-1])
      dt[,,1] <- 1
      dt <- as.vector(dt)[array(1:prod(ddd), ddd)
                          [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor), ]]
      if (!is.null(bor) && bor>0) {
        X <- X[array(1:prod(ddd), ddd)
               [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor), ], ]
        if (is.matrix(y))
          y <- y[array(1:prod(ddd), ddd)
                 [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor), ], ]
        else
          y <- y[array(1:prod(ddd), ddd)
                 [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor), ]]
      }
    }
    else {
      if (!is.null(bor) && bor>0) {
        X <- X[array(1:prod(ddd), ddd)
               [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor)], ]
        if (is.matrix(y))
          y <- y[array(1:prod(ddd), ddd)
                 [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor)], ]
        else
          y <- y[array(1:prod(ddd), ddd)
                 [(bor+1):(ddd[1]-bor), (bor+1):(ddd[2]-bor)]]
      }
    }
  }
  if (dtemporal & onlynew) {
    if (!is.matrix(y.ori))
      y.ori <- cbind(y.ori,1)
    yy <- tapply(y.ori[,1],
                 lapply(data[,c(rcol,ccol,tcol)],
                        as.character), as.numeric)
    for (i in 2:ddd[3])
      yy[,,i] <- ifelse(yy[,,i-1]>0, NA, yy[,,i])
    if (is.matrix(y))
      y[,1] <- as.vector(yy)[array(1:prod(ddd), ddd)
                           [(1+bor):(ddd[1]-bor), (1+bor):(ddd[2]-bor), ]]
    else
      y <- as.vector(yy)[array(1:prod(ddd), ddd)
                         [(1+bor):(ddd[1]-bor), (1+bor):(ddd[2]-bor), ]]
  }
  if (is.matrix(y)) {
    id <- !(apply(is.na(X), 1, any) | apply(is.na(y),1,any))
    X <- X[id,]
    y <- y[id,]
    dt <- dt[id]
  }
  else {
    id <- !(apply(is.na(X), 1, any) | is.na(y))
    X <- X[id,]
    y <- y[id]
    dt <- dt[id]
  }
  if (length(ddd)==2) {
    stop("In construction for spatial, please see old versions...")
  }
  if (length(ddd)==3) {
    res <- list(naive=glm.fit(x=X, y=y, family=family()))
    class(res$naive) <- "glm"
    if (brlr) {
      if (family()$family!="binomial") 
        stop("Only binomial family implemented for Bias Reduction!")
      if (family()$link!="logit") 
        warning("Only logit link implemented for Bias Reduction.")
      opt <- optim(coef(res$naive), nllbrlr,
                   method="BFGS", hessian=TRUE)
      res$coefficients <- opt$par
      res$linear.predictors <- X%*%opt$par
      res$fitted.values <- family()$linkinv(res$linear.predictors)
      res$residuals <- y-res$fitted.values
      res$logLik <- sum(dbinom(y, 1, res$fitted.values, log=TRUE))
      res$npars <- ncol(X)
      res$df.null <- length(y)
      res$df.residual <- res$df.null-res$npars
      res$deviance <- -2*res$logLik
      res$null.deviance <-
        -2*sum(dbinom(y, 1, fitted(glm(y~1,binomial)), log=TRUE))
      res$family <- family
      res$optim.res <- opt
    }
  }
  class(res) <- "aglm"
  return(res)
}

