\name{aglm}
\alias{aglm}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Fit a spatio-temporal autoregressive model for lattice data}
\description{
  This function fit's a spatio-temporal autoregressive model for
  lattice data. At the moment only implemented the binomial family,
  the autologistic model.
}
\usage{
aglm(formula, family = gaussian, data,  rcol=NULL, ccol=NULL, 
     tcol=NULL, nb, bor=NULL, onlynew=FALSE, weights, 
     start = NULL, brlr = TRUE, nboot = 999,  ...)
}
\arguments{
  \item{formula}{a symbolic description of the model to be fit.
  See details im \code{formula} help}
  \item{family}{a description of the error distribution and link
    function to be used in the model. This can be a character
    string naming a family function, a family function or the
    result of a call to a family function.  (See 'family' for
    details of family functions.)}
  \item{data}{an object of class \code{data.frame}}
  \item{rcol}{column of data with row coordinates}
  \item{ccol}{column of data with col coordinates}
  \item{tcol}{column of data with temporal coordinates} 
  \item{nb}{object of \cote{nb} class defined in spdep package}
  \item{bor}{number of borders excluded for response} 
  \item{onlynew}{if the analisys is for only new incidences over time}
  \item{weights}{an optional vector of weights to be used in the fitting
         process.  Should be 'NULL' or a numeric vector.}
  \item{start}{starting values for the parameters in the linear predictor.}
  \item{brlr}{Use the bias-reduced logistic regresssion}
  \item{nboot}{The number of bootstraps via Gibbs Sampler for
    variance estimation}
}
\details{
  In this function, the formula has similar use in glm function, but 
  contain terms with spatial and temporal lag terms. The expression 
  for lags is int the form \code{stLag(spform, tLag)}. The \code{spform} 
  is the formula symbolic expression to spatial lag terms and the 
  \code{tLag} is a integer non positive to temporal lag. All possible 
  terms terms implemented are explained in next paragraph.

  \code{stLag(y ~ S + R + C + first + dA + dB + D + second,k)}, where 
  \item{S}{same spatial position, reasonable to use if k is negative}
  \item{R}{neighbours position at same row}
  \item{C}{neighbours position at same column}
  \item{first}{neighbours at same row and column or the neighbours of 
    each area informed in \code{nb} argument in non regular lattice case}
  \item{dA}{neighbours at principal diagonal positions}
  \item{dB}{neighbours at secondary diagonal positions}
  \item{D}{neighbours at both diagonal positions}
  \item{second}{neighbours at first position plus neighbours at diagonal 
    positions or neighbours of neighbours in non regular lattice case}
  \item{k}{is the temporal lag, a non positive integer}
}
\value{
  An list with two possible results:
  If the spatial models only \code{naive}, \code{boot} and \code{mc} 
  itens. If a spatio-temporal model not \code{boot} and \code{mc} itens
  
  \item{naive}{The result returned by glm() function or brlr() function}
  \item{boot}{A matrix nboot times number of parameters}
  \item{mc}{A matrix nboot times number of parameters}
  \item{coefficients}{a named vector of coefficients}
  \item{linear.preditors}{the linear fit on link scale}
  \item{fitted.values}{the fitted mean values, obtained by transforming the
    linear predictors by the inverse of the link function}
  \item{residuals}{the residuals}
  \item{logLik}{the log likelihood}
  \item{npars}{number of parameters in the model}
  \item{df.null}{the residual degrees of freedom for the null model}
  \item{df.residual}{the residual degrees of freedom}
  \item{deviance}{minus twice the maximized log-likelihood}
  \item{null.deviance}{The deviance for the null model, comparable with
    \code{deviance}}
  \item{optim.res}{The results obtained by optim() funciton for
    spatio-temporal estimation with time irregular inervals
    between evaluations}
}
\references{ ~put references to the literature/web site here ~ }
\author{Elias T. Krainski and Paulo J. Ribeiro Jr.}
\note{to do}
\seealso{See Also as \code{\link{glm}} }
\examples{

### autologistic spatio-temporal model

data(valencia)

### analisys of evaluations after evaluation number 23
map <- as.numeric(substring(as.character(valencia$mapa),5))
id.sel <- which(map>=23)


m1 <- aglm(inplsifr~inplacra+stLag(inplsifr~R,-1), binomial,
           valencia[id.sel,], rcol=4, ccol=5,tcol=22)
summary(m1)

m2 <- aglm(inplsifr~inplacra+stLag(inplsifr~R,-1)+stLag(inplacra~S,-1)+
           stLag(inplacra~S,-2) + stLag(inplacra~S,-3), binomial,
           valencia[id.sel,], rcol=4, ccol=5,tcol=22)
summary(m2)

}
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
